; WALK.asm
; --------------------------------------------------------
;   Example for XESS board 8031, linked with VTOP FPGA
; --------------------------------------------------------
; Bertrand Cuzeau
; alse@compuserve.com
;
; This works as follows:
;  - a bit is walked four times on the LED display
;  - a seed is then written in the LFSR
;  - the LFSR is shifted 32 times, read back,
;    and written to the LED
;  - the system cycles back to the beginning of the test
;
; This is probably not a very elegant code, but I am
; not familiar with the 8031...
;
; Don't forget to Pulse D0 in the parallel port (reset) !

        $MOD51

STACKTOP EQU    70H           ; start of stack (grows up)
LEDREG   EQU    0F000H        ; LED register address
LFSR     EQU    0E000H        ; LFSR R/W register address
LFShift  EQU    0D000H        ; LFSR shift command

        DSEG
        ORG     30H
CNTR:   DS      1             ; counter for wait routine
CNT2:   DS      1             ; counter for bit walk test

        CSEG
        ORG     0000H         ; program starts at 0 after reset

START:
        MOV     SP,#STACKTOP  ; initialize stack pointer ...

; --- Walking bit LED Test ---
START2:
        MOV     A,#1          ; A(0) <= 1
        MOV     DPTR,#LEDREG  ; point on FPGA LED register
        MOV     CNT2,#32      ; 4 passes
LOOP:
        RL      A
        MOVX    @DPTR,A       ; show A the LED digit
        CALL    WAIT          ; delay so we can see the bits ...
        DJNZ    CNT2,LOOP

; --- LFSR Test ---

        MOV     A,#1          ; Seed
        MOV     DPTR,#LFSR    ; point on E000=FPGA LFSR register
        MOV     CNT2,#32      ; 32 passes
        MOVX    @DPTR,A       ; Store Seed
LOOP1:
        MOV     DPTR,#LFSR    ; point on E000=FPGA LFSR register
        MOVX    A,@DPTR       ; Read LFSR
        MOV     DPTR,#LEDREG  ; point on FPGA LED register
        MOVX    @DPTR,A       ; Write to the LED digit
        CALL    WAIT          ; delay so we can see the bits ...
        MOV     DPTR,#LFShift
        MOVX    @DPTR,A       ; Dummy write to shift
        DJNZ    CNT2,LOOP1

; --- STOP ---
        JMP     START2
        JMP     $             ; Suicide... (wait for a reset)

; This subroutine waits about 1/8th second
WAIT:
        PUSH    ACC
        PUSH    B
        MOV     CNTR,#2       ; 1 = ~ 1/8th sec
WAIT1:
        MOV     B,#255
WAIT2:
        MOV     A,#255
        DJNZ    ACC,$
        DJNZ    B,WAIT2
        DJNZ    CNTR,WAIT1
        POP     B
        POP     ACC
        RET

        END

